/** gomad.c - import areas from Squish.Cfg to Med.Fld

    author: Cyril Slobin (2:5020/358.19, 2:5020/219.44)
    based on txt2med.c by Pete Kvitek (2:5020/6) et al.

    version 0.99.2

    compiled with Turbo C 2.0

    Made on Earth
    Public Domain
*/

#define SOFTWARE_AUTHOR "Cyril Slobin (2:5020/358.19, 2:5020/219.44)"
#define SOFTWARE_VERSION "0.99.2"

/* Following comments are in Russian language, codepage 866 is used. */

/*  ணࠬ  ᠭ ⥩  䠩 䨣樨
   ᪢ (Squish.Cfg)  䠩 ᠭ 䮫஢  (Med.Fld).

    :

   1. ᫨  䨣 (䠩 Gomad.Cfg  ⥪饬 ⠫   ⮬ 
      ⠫,   ᠬ ணࠬ),  뢠  ࠡ뢠
      (㭪 process_gomad_cfg);
   2.   䠩 Mad.Tmp;
   3. ᫨   䠩 ᠭ 䮫஢ (Med.Fld),  뢠
            Med.Tmp, ⭮  
      㯮   ॠ⠣ (㭪 process_med_cfg);
   4. 뢠 䠩 䨣樨 ᪢ (Squish.Cfg), ᠭ  
      , ᫨    뫮  Med.Fld,   Med.Tmp
      (㭪 process_squish_cfg);
   5.  Med.Fld २뢠  Med.Bak, Med.Tmp -  Med.Fld.
*/

/* ⠭ . */

#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*   ᠭ  Med.Fld - ࠭  Txt2med.C */

#include "madmed.h"

/*  error 뢠  ⠫ 訡; ॠ쭠 ࠡ
     㭪樨 _error,   㦭, ⮡ ।  
   ⥪饣 䠩   ப  .
*/

#define error(message) _error(__FILE__, __LINE__, message)

/* ⢥ ࠭祭: ᨬ쭠   䠩,
   ᨬ쭮 ⢮ 宮⥩,  ୮ 
   (࠭稢  ६    Med.Fld), 
   ⥪⮢  (࠭稢  ப  *.cfg 䠩).
*/

#define MAX_FILENAME_LENGTH 256
#define MAX_AREAS_DEFINED 4096
#define MAX_BINARY_BUFFER 4096
#define MAX_TEXT_BUFFER 4096

/* MAGIC    CRC-32,  SP -  ஡ . */

#define MAGIC 0xEDB88320uL
#define SP " \t"

/*  ⨯ ⥩ ( passthru)   . */

enum AREATYPE { NETAREA, BADAREA, DUPEAREA, ECHOAREA, PASSTHRU };
enum ERRORLEVEL { ZERO = 0, SUCCESS = 1, WARNING = 2, ERROR = 3 };

/*   AREA ࠭  ଠ  ,  
   ௭  Squish.Cfg.
*/

typedef struct _AREA {
    int areatype;
    int basetype;
    char *areatag;
    char *basepath;
    NETADDR address;
} AREA;

/*   ᨢ  5 ⮢  ;   㬮砭 
    ⨯ ⥩, ᫥  enum AREATYPE.

     default_explicit 砥,  ⥢  
   ⥩ ⮣ ⨯  ய뢠,  ᫨  ᮢ 
   ᭮ ᮬ.   default_ignore 砥,   ⨯
   ⥩  㬮砭  (砫쭮 passthru).
   default_flags  default_attrs - 䫠  ਡ  㬮砭.
*/

static int default_explicit[5] = { 0, 0, 0, 0, 0 };
static int default_ignore[5] = { 0, 0, 0, 0, 1 };

static USHORT default_flags[5] = {
    FF_NETMAIL | FF_ADDORIGIN,
    FF_LOCALMAIL,
    FF_LOCALMAIL,
    FF_ECHOMAIL | FF_ADDORIGIN,
    FF_ECHOMAIL | FF_ADDORIGIN,
};

static ULONG default_attrs[5] = {
    MSGPRIVATE | MSGKILL | MSGLOCAL,
    MSGLOCAL,
    MSGLOCAL,
    MSGLOCAL,
    MSGLOCAL,
};

/*   ⠡ ᮮ⢥⢨ ⥪⮢  䫠  ਡ⮢ 
    ७ ।⠢ﬨ; ᯮ  ⥭ Gomad.Cfg.
*/

static struct { const char *name; USHORT flag; } flag_names[] = {
    { "TwitFolder", FF_TWITFOLDER },
    { "AddINTL", FF_ADDINTL },
    { "AddDOMAIN", FF_ADDDOMAIN },
    { "AddOrigin", FF_ADDORIGIN },
    { "RandOrigin", FF_RANDORIGIN },
    { "ShowTwits", FF_SHOWTWITS },
    { "NoSentMail", FF_NOSENTMAIL },
    { NULL, 0 }
};

static struct { const char *name; ULONG attr; } attr_names[] = {
    { "Private", MSGPRIVATE },
    { "KillSnt", MSGKILL },
    { "Crash", MSGCRASH },
    { "Receipt", MSGRRQ },
    { "Hold", MSGHOLD },
    { "Local", MSGLOCAL },
    { NULL, 0 }
};

/* areapool -  ᯨ᮪ 襩  ⨢ ॠ⠣;
   crctable - ᯮ⥫쭠 ⠡   CRC-32.
*/

static unsigned long areapool[MAX_AREAS_DEFINED];
static unsigned long crctable[256];

/*  ६ ᯮ  ࠡ⪨ ⠫ 訡 
   ࠧ Gomad.Cfg  Squish.Cfg;

   warn_filename -  ࠡ뢠 䠩,
   warn_line -  ࠡ뢠 ப,
   warn_counter - 稪 訡.
*/

static const char *warn_filename;
static int warn_line;
static int warn_counter;

/* primary - ᭮  ( ࢮ 窨 Address  *.Cfg),
   known_primary - 뢠,   㦥 ⠭ ᭮ .
*/

static NETADDR primary;
static int known_primary;

/* 稪   ⥩. */

static int area_counter;

/* 㭪 _error 뢠  ⠫ 訡;  error ⮢
       㬥 -  䠩 ணࠬ   ப, 
   ன 稫 訡; ⨩ 㬥 - ᮮ饭  訡.
*/

void _error(const char *file, int line, const char *message)
{
    fprintf(stderr, "%s (%s %d)\n", message, file, line);
    exit(ERROR);
}

/* 㭪 exist_file ஢ ⢮ 䠩. */

int exist_file(const char *name)
{
    return !access(name, 0);
}

/* 㭪 open_file 뢠 䠩  ࠡ뢠  訡. */

FILE *open_file(const char *name, const char *mode)
{
    FILE *file = fopen(name, mode);
    if (file == NULL) {
        fprintf(stderr, "Can\'t open file %s (%s): %s", name, mode, strerror(errno));
        exit(ERROR);
    }
    return file;
}

/* 㭪 close_file 뢠 䠩  ࠡ뢠  訡. */

void close_file(FILE *file, const char *name)
{
    int xx = fclose(file);
    if (xx != 0) {
        fprintf(stderr, "Can\'t close file %s: %s", name, strerror(errno));
        exit(1);
    }
}

/* 㭪 rightmost 頥 㪠⥫  ᠬ ࠢ 宦  s
    ᨬ  t  NULL, ᫨   ᨬ  t  s  室;
   祭 宦  ⠭ strpbrk, ⮫쪮  ࠢ,   ᫥.
*/

char *rightmost(const char *s, const char *t)
{
    const char *p;

    for (p = s + strlen(s) - 1; p >= s; p--) {
        if (strchr(t, *p) != NULL) return (char *)p;
    }
    return NULL;
}

/* 㭪 basename 뢠       䠩,
   ⠢ ᮡ⢥   ७. 頥 㪠⥫ 
   । ண ,  ஢  ந室.
*/

const char *basename(const char *name)
{
    const char *p = rightmost(name, "\\/");

    return p != NULL ? p + 1 : name;
}

/* 㭪 replace_basename ᮧ   䠩, 室饣  ⮬
    ⠫,   old,    basename.
*/

void replace_basename(char *new, const char *old, const char *basename)
{
    const char *p = rightmost(old, "\\/");
    size_t pathlen = p ? p - old + 1 : 0;

    if (pathlen + strlen(basename) >= MAX_FILENAME_LENGTH) error("Filename too long");

    strncpy(new, old, pathlen);
    strcpy(new+pathlen, basename);
}

/* 㭪 replace_extension ᮧ   䠩, ᮢ饥  old
    ᥬ, ஬ ७, ஥   ext.
*/

void replace_extension(char *new, const char *old, const char *ext)
{
    const char *p = strrchr(old, '.');
    size_t baselen = p ? p - old : strlen(old);

    if (baselen + strlen(ext) >= MAX_FILENAME_LENGTH) error("Filename too long");

    strncpy(new, old, baselen);
    strcpy(new + baselen, ext);
}

/* 㭪 getline 뢠   s  n ப  䠩 f,  
    n-1 ᨬ,  蠥  ᨬ \0.  \n   
   . ᫨ ப 뫠  n-1 ᨬ, ⮪ 뢠
    모뢠.  ⠭ fgets,   ᮢᥬ.
*/

char *getline (char *s, size_t n, FILE *f)
{
    size_t i;
    char *p;
    int c;

    if (s == NULL || n <= 0 || f == NULL) return NULL;
    for (i = 1, p = s; i < n; i++, p++) {
        c = getc(f);
        if (c == '\n' || c == EOF) break;
        *p = c;
    }
    if (p == s && c == EOF) return NULL;
    *p = '\0';
    while (c != '\n' && c != EOF) {
        c = getc(f);
    }
    return s;
}

/* 㭪 parse_address ࠧࠥ ப, ᮤঠ FTN , 
   頥   ᯥ    㤠; ᠬ   
    NETADDR ( ⪨ 楫)   ᯥ 
    ண 㬥. ᫨  ,  / ॣ
     쥣 㬥; ᫨ ⨩ 㬥 - NULL, 
     .
*/

int parse_address(const char *s, NETADDR *address, NETADDR *primary)
{
    unsigned short zone, net, node, point;
    char c;

    if (sscanf(s, "%hd:%hd/%hd.%hd%c", &zone, &net, &node, &point, &c) == 4) {
        address->zone = zone;
        address->net = net;
        address->node = node;
        address->point = point;
        return 1;
    } else if (sscanf(s, "%hd:%hd/%hd%c", &zone, &net, &node, &c) == 3) {
        address->zone = zone;
        address->net = net;
        address->node = node;
        address->point = 0;
        return 1;
    } else if (primary == NULL) {
        return 0;
    } else if (sscanf(s, "%hd/%hd.%hd%c", &net, &node, &point, &c) == 3) {
        address->zone = primary->zone;
        address->net = net;
        address->node = node;
        address->point = point;
        return 1;
    } else if (sscanf(s, "%hd/%hd%c", &net, &node, &c) == 2) {
        address->zone = primary->zone;
        address->net = net;
        address->node = node;
        address->point = 0;
        return 1;
    } else if (sscanf(s, "%hd.%hd%c", &node, &point, &c) == 2) {
        address->zone = primary->zone;
        address->net = primary->net;
        address->node = node;
        address->point = point;
        return 1;
    } else if (sscanf(s, "%hd%c", &node, &c) == 1) {
        address->zone = primary->zone;
        address->net = primary->net;
        address->node = node;
        address->point = 0;
        return 1;
    } else {
        return 0;
    }
}

/* 㭪 warn 뤠 ᮮ饭  ⠫쭮 訡  ࠧ 䠩
   䨣樨 (Gomad.Cfg  Squish.Cfg). ᯮ 
   ६ warn_filename  warn_line, 㢥稢 warn_counter.
*/

void warn(const char *message)
{
    fprintf(stderr, "%s %d: %s\n", warn_filename, warn_line, message);
    ++warn_counter;
}

/* 㭪 search_flag  䫠   ⥪⮢ ; 頥
     ᯥ,   㤠, ᠬ 䫠 뢠  
   ண ࠬ.
*/

int search_flag(const char *p, USHORT *flag)
{
    int i;

    for (i = 0; flag_names[i].name != NULL; i++) {
        if (stricmp(p, flag_names[i].name) == 0) {
            *flag = flag_names[i].flag;
            return 1;
        }
    }
    return 0;
}

/* 㭪 search_attr  ਡ   ⥪⮢ ; 頥
     ᯥ,   㤠, ᠬ ਡ 뢠 
    ண ࠬ.
*/

int search_attr(const char *p, ULONG *attr)
{
    int i;

    for (i = 0; attr_names[i].name != NULL; i++) {
        if (stricmp(p, attr_names[i].name) == 0) {
            *attr = attr_names[i].attr;
            return 1;
        }
    }
    return 0;
}

/* 㭪 process_gomad_cfg ࠡ뢠 䠩 䨣樨 Gomad.Cfg.
     ࠡ - 䨪  ᨢ default_ignore,
   default_flags  default_attrs. ᫨  䠩 뫠 ४⨢ Address,
   ⠭ primary  known_primary.    砥
   ࠧࠥ  , , ᫨   뫠    訡 (warn),
   뢠 訡 ⠫쭠 (error)   ⮬  稢.
*/

void process_gomad_cfg(FILE *gomad_cfg, const char *filename)
{
    char buffer[MAX_TEXT_BUFFER];
    int sign, areatype;
    char *p;
    USHORT flag;
    ULONG attr;

    warn_filename = filename;
    warn_line = 0;
    warn_counter = 0;

    known_primary = 0;

    while (getline(buffer, MAX_TEXT_BUFFER, gomad_cfg)) {
        ++warn_line;

        p = strtok(buffer, SP);
        if (p == NULL) continue;
        if (*p == ';') continue;

        if (stricmp(p, "Address") == 0) {
            if (known_primary) {
                warn("Primary address redefined");
                continue;
            }
            p = strtok(NULL, SP);
            if (!parse_address(p, &primary, NULL)) {
                warn("Bad primary address");
                continue;
            }
            known_primary = 1;
            continue;
        }

        if (stricmp(p, "NetArea") == 0) {
            areatype = NETAREA;
        } else if (stricmp(p, "BadArea") == 0) {
            areatype = BADAREA;
        } else if (stricmp(p, "DupeArea") == 0) {
            areatype = DUPEAREA;
        } else if (stricmp(p, "EchoArea") == 0) {
            areatype = ECHOAREA;
        } else if (stricmp(p, "PassThru") == 0) {
            areatype = PASSTHRU;
        } else {
            warn("Unknown area type");
            continue;
        }

        default_ignore[areatype] = 0;

        while (1) {
            p = strtok(NULL, SP);
            if (p == NULL) break;
            if (*p == '+' || *p == '-') {
                sign = *p;
            } else {
                warn("Flag not signed");
                continue;
            }
            ++p;
            if (*p == NULL) {
                warn("Empty flag");
                continue;
            }
            if (stricmp(p, "Explicit") == 0) {
                default_explicit[areatype] = (sign == '+');
                continue;
            }
            if (stricmp(p, "Ignore") == 0) {
                default_ignore[areatype] = (sign == '+');
                continue;
            }
            if (search_flag(p, &flag)) {
                if (sign == '+') default_flags[areatype] |= flag;
                else default_flags[areatype] &= ~flag;
                continue;
            }
            if (search_attr(p, &attr)) {
                if (sign == '+') default_attrs[areatype] |= attr;
                else default_attrs[areatype] &= ~attr;
                continue;
            }
            warn("Unknown flag");
        }
    }
    if (ferror(gomad_cfg)) error("Gomad config file corrupt");
    if (warn_counter > 0) error("Errors in gomad config file");
}

/* 㭪 initialize_crctable  ⠡   CRC-32. */

void initialize_crctable(void)
{
    int i, j;
    unsigned long t;

    for (i = 0; i < 256; i++) {
        t = i;
        for (j = 0; j < 8; j++) {
            t = (t & 1) ? (t >> 1) ^ MAGIC : (t >> 1);
        }
        crctable[i] = t;
    }
}

/* 㭪 initialize_areapool  ᯨ᮪  ⥩. */

void initialize_areapool(void)
{
    int i;

    for (i = 0; i < MAX_AREAS_DEFINED; i++) {
        areapool[i] = 0;
    }
}

/* 㭪 hash 頥 ᢥ CRC-32  ᢮ 㬥. */

unsigned long hash(const char *p)
{
    unsigned long crc;
    char c;

    crc = ~0uL;
    while ((c = *p++) != '\0') {
        crc = (crc >> 8) ^ crctable[(unsigned char)(crc ^ c)];
    }
    crc = ~crc;
    return crc;
}

/* 㭪 already_defined 頥 , ᫨  ⠪ 㬥⮬ 
    㦥 頫,  , ᫨     ࠧ;  ஬
   砥 㬥 ,   ᫥騩 ࠧ   ୥
   㦥 . 㬥    CRC-32 ᢥ⮪  areapool.
*/

int already_defined(const char *a)
{
    int i;
    unsigned long code = hash(a);

    for (i = 0; i < MAX_AREAS_DEFINED; i++) {
        if (areapool[i] == code) return 1;
        if (areapool[i] == 0) { areapool[i] = code; return 0; }
    }
    error("Too many areas defined");
    return -1;
}

/* 㭪 process_med_fld  䠩 Med.Fld  Med.Tmp  
   , ⭮   areapool ॠ⠣  㯮 
   Med.Fld ⥩.  ⠫ 訡  ਢ Med.Fld 
    ஡  -뢮.
*/

void process_med_fld(FILE *med_fld, FILE *med_tmp)
{
    char buffer[MAX_BINARY_BUFFER];
    FLDT folder;
    size_t n, varsize;
    char *areaname;

    while ((n = fread(&folder, 1, sizeof(folder), med_fld)) != 0) {
        if (n < sizeof(folder)) error("Folder list file corrupt");
        varsize = folder.cchName + folder.cchArea + folder.cchPath + folder.cchOrig + folder.cchTemp;
        if (varsize >= MAX_BINARY_BUFFER) error("Folder list file corrupt");
        n = fread(buffer, 1, varsize, med_fld);
        if (n < varsize) error("Folder list file corrupt");
        n = fwrite(&folder, 1, sizeof(folder), med_tmp);
        if (n < sizeof(folder)) error("Can\'t write to temporary file");
        n = fwrite(buffer, 1, varsize, med_tmp);
        if (n < varsize) error("Can\'t write to temporary file");
        areaname = buffer + folder.cchName;
        areaname[folder.cchArea] = '\0';
        strupr(areaname);
        already_defined(areaname);
    }
    if (ferror(med_fld)) error("Folder list file corrupt");
}

/* 㭪 parse_squish_cfg_line ࠧࠥ ப 䠩 䨣樨
   ᪢. 頥 , ᫨ ப ᮤন ࠢ쭮 ᠭ
   ,  ; ᠬ ᠭ     AREA
     ண 㬥. ᫨ ப ᮤঠ ᭮ , 
      ६ primary   known_primary 
    (ᠬ 㭪  ⮬ 頥 ).  ⠫
   (᭮  宩       । )
    ⠫ (ਢ ᠭ ) 訡.
 */

int parse_squish_cfg_line(char *s, AREA *area)
{
    char *p, *areatag, *basepath;
    int areatype, basetype;
    NETADDR address;
    int address_defined;

    p = strtok(s, SP);
    if (p == NULL) return 0;
    if (*p == ';') return 0;

    if (stricmp(p, "Address") == 0) {
        if (known_primary) return 0;
        p = strtok(NULL, SP);
        if (!parse_address(p, &primary, NULL)) {
            warn("Bad primary address");
            error("Fatal error");
        }
        known_primary = 1;
        return 0;
    }

    if (stricmp(p, "NetArea") == 0) {
        areatype = NETAREA;
    } else if (stricmp(p, "BadArea") == 0) {
        areatype = BADAREA;
    } else if (stricmp(p, "DupeArea") == 0) {
        areatype = DUPEAREA;
    } else if (stricmp(p, "EchoArea") == 0) {
        areatype = ECHOAREA;
    } else {
        return 0;
    }

    if (!known_primary) {
        warn("No primary address defined");
        error("Fatal error");
    }

    p = strtok(NULL, SP);
    if (p == NULL) { warn("No areatag"); return 0; }
    areatag = p;
    strupr(areatag);

    p = strtok(NULL, SP);
    if (p == NULL) { warn("No basepath"); return 0; }
    basepath = p;
    strupr(basepath);

    basetype = FF_MSGSDM;
    address.zone = 0;
    address.net = 0;
    address.node = 0;
    address.point = 0;
    address_defined = 0;

    while (1) {
        p = strtok(NULL, SP);
        if (p == NULL) break;
        if (*p++ != '-') break;
        switch (*p) {
        case '$':
            basetype = FF_MSGSQUISH;
            continue;
        case '0':
            if (areatype != ECHOAREA) {
                warn("Bad flag for this area type");
                return 0;
            }
            areatype = PASSTHRU;
            continue;
        case 'p':
            if (address_defined) {
                warn("Address redefined");
                return 0;
            }
            address_defined = 1;
            if (!parse_address(p+1, &address, &primary)) {
                warn("Bad area address");
                return 0;
            }
            continue;
        }
    }

    area->areatype = areatype;
    area->basetype = basetype;
    area->areatag = areatag;
    area->basepath = basepath;
    if (default_explicit[areatype] || memcmp(&address, &primary, sizeof(address)) != 0) {
        area->address = address;
    } else {
        memset(&area->address, '\0', sizeof(address));
    }

    return 1;
}

/* 㭪 process_squish_cfg ࠡ뢠 䠩 Squish.Cfg  뢠
   ।      䠩 Med.Tmp;  ⠫
   訡  砥 ஡  -뢮.
*/

void process_squish_cfg(FILE *squish_cfg, FILE *med_tmp, const char *filename)
{
    char buffer[MAX_TEXT_BUFFER];
    AREA area;
    FLDT folder;
    size_t n, taglength, pathlength;
    int addbackslash;

    warn_filename = filename;
    warn_line = 0;
    warn_counter = 0;

    area_counter = 0;

    while (getline(buffer, MAX_TEXT_BUFFER, squish_cfg)) {
        ++warn_line;
        if (parse_squish_cfg_line(buffer, &area)) {
            if (!already_defined(area.areatag)) {
                if (default_ignore[area.areatype]) continue;
                memset(&folder, '\0', sizeof(folder));
                taglength = strlen(area.areatag);
                pathlength = strlen(area.basepath);
                addbackslash = (area.basetype == FF_MSGSDM && area.basepath[pathlength-1] != '\\');
                folder.cb = sizeof(folder) + 2 * taglength + pathlength + addbackslash;
                folder.fs = default_flags[area.areatype] | area.basetype;
                folder.cchName = taglength;
                folder.cchArea = taglength;
                folder.cchPath = pathlength + addbackslash;
                folder.netAddr = area.address;
                folder.attr = default_attrs[area.areatype];
                n = fwrite(&folder, 1, sizeof(folder), med_tmp);
                if (n < sizeof(folder)) error("Can\'t write to temporary file");
                n = fwrite(area.areatag, 1, taglength, med_tmp);
                if (n < taglength) error("Can\'t write to temporary file");
                n = fwrite(area.areatag, 1, taglength, med_tmp);
                if (n < taglength) error("Can\'t write to temporary file");
                n = fwrite(area.basepath, 1, pathlength, med_tmp);
                if (n < pathlength) error("Can\'t write to temporary file");
                if (addbackslash) {
                    n = fwrite("\\", 1, 1, med_tmp);
                    if (n < 1) error("Can\'t write to temporary file");
                }
                puts(area.areatag);
                ++area_counter;
            }
        }
    }
    if (ferror(squish_cfg)) error("Squish config file corrupt");
}

/* ਩   ணࠬ ਢ  ᠬ 砫  ࠧ
   " ".
*/

void main(int argc, char *argv[])
{
    const char *squish_cfg_filename, *med_fld_filename;
    char gomad_cfg_filename[MAX_FILENAME_LENGTH];
    char med_tmp_filename[MAX_FILENAME_LENGTH];
    char med_bak_filename[MAX_FILENAME_LENGTH];
    FILE *gomad_cfg, *squish_cfg, *med_fld, *med_tmp;

    puts("gomad v"SOFTWARE_VERSION" ("__DATE__") by "SOFTWARE_AUTHOR);
    if (argc != 3) {
        puts("Usage: gomad <path to Squish.Cfg> <path to Med.Fld>");
        exit(ERROR);
    }

    if (exist_file("gomad.cfg")) {
        gomad_cfg = open_file("gomad.cfg", "rt");
        process_gomad_cfg(gomad_cfg, "gomad.cfg");
        close_file(gomad_cfg, "gomad.cfg");
    } else {
        replace_basename(gomad_cfg_filename, argv[0], "gomad.cfg");
        if (exist_file(gomad_cfg_filename)) {
            gomad_cfg = open_file(gomad_cfg_filename, "rt");
            process_gomad_cfg(gomad_cfg, "gomad.cfg");
            close_file(gomad_cfg, gomad_cfg_filename);
        }
    }

    squish_cfg_filename = argv[1];
    med_fld_filename = argv[2];

    replace_extension(med_tmp_filename, med_fld_filename, ".tmp");
    replace_extension(med_bak_filename, med_fld_filename, ".bak");

    remove(med_tmp_filename);
    remove(med_bak_filename);

    initialize_crctable();
    initialize_areapool();

    squish_cfg = open_file(squish_cfg_filename, "rt");
    med_tmp = open_file(med_tmp_filename, "wb");

    if (exist_file(med_fld_filename)) {
        med_fld = open_file(med_fld_filename, "rb");
        process_med_fld(med_fld, med_tmp);
        close_file(med_fld, med_fld_filename);
    }

    process_squish_cfg(squish_cfg, med_tmp, basename(squish_cfg_filename));

    close_file(squish_cfg, squish_cfg_filename);
    close_file(med_tmp, med_tmp_filename);

    rename(med_fld_filename, med_bak_filename);
    rename(med_tmp_filename, med_fld_filename);

    exit(warn_counter > 0 ? WARNING : area_counter > 0 ? SUCCESS : ZERO);
}
