/** madmed.h - block of definitions torn out from txt2med.h */

 /* Standard typed overtyping */

 typedef unsigned short USHORT;
 typedef unsigned long  ULONG;
 typedef unsigned char  BYTE;
 typedef unsigned int   BOOL;

 /* Some useful declarations */

 #define loop for (;;)

 #define TRUE   1
 #define FALSE  0

 /* Netword address structure */

 typedef struct _NETADDR {
   USHORT zone;
   USHORT net;
   USHORT node;
   USHORT point;
 } NETADDR;

  /* Folder template structure. Note: folder definition file is */
  /* an array of the variable length folder template structures */

  typedef struct _FLDT {        /* fldt */
    USHORT  cb;                 /* Size of this structure */
    USHORT  fs;                 /* Folder flags (FF_) */
    USHORT  iLevel;             /* Folder level index */
    USHORT  cchName;            /* Folder name length */
    USHORT  cchArea;            /* Folder area length */
    USHORT  cchPath;            /* Folder msgbase path length */
    USHORT  cchOrig;            /* Folder origin string length */
    USHORT  cchTemp;            /* Folder template path length */
    NETADDR netAddr;            /* Default originating address */
    ULONG   attr;               /* Default message attributes (MA_) */
    USHORT  boardno;            /* Hudson msgbase board number */
    BYTE    reserved[26];       /* Reserved for future extensions */
    USHORT  fs3rdParty;         /* Third party program flags (FF3_) */
/* */
/*  Following strings are stored without trailing null */
/* */
/*  CHAR    achName[cchName];   // Folder name */
/*  CHAR    achArea[cchArea];   // Folder area */
/*  CHAR    achPath[cchPath];   // Folder msgbase path */
/*  CHAR    achOrig[cchOrig];   // Folder origin string */
/*  CHAR    achTemp[cchTemp];   // Folder template path */
  } FLDT, * PFLDT;

  /* Folder flags in fldt.fs */

#define FF_XMSGBASE      0x000f /* Msg base type mask */
#define FF_MSGSDM        0x0000 /* FTCS-001 *.MSG message base */
#define FF_MSGSQUISH     0x0001 /* Squish *.SQ? message base */
#define FF_MSGJAM        0x0002 /* JAM *.J?? message base */
#define FF_MSGHUDSON     0x0003 /* Hudson message base */

#define FF_XFLDTYPE      0x0070 /* Folder type mask */
#define FF_NETMAIL       0x0010 /* Netmail folder */
#define FF_ECHOMAIL      0x0020 /* Echomail folder */
#define FF_GROUPMAIL     0x0040 /* Group mail folder */
#define FF_LOCALMAIL     0x0000 /* Local mail folder */

#define FF_COLLAPSED     0x0080 /* Subfolders collapsed */
#define FF_TWITFOLDER    0x0100 /* Twit folder */
#define FF_ADDINTL       0x0200 /* Add ^aINTL kludge */
#define FF_ADDDOMAIN     0x0400 /* Add ^aDOMAIN kludge */
#define FF_ADDORIGIN     0x0800 /* Add origin line */
#define FF_RANDORIGIN    0x1000 /* Random origin line */
#define FF_SHOWTWITS     0x2000 /* Always show twit messages */
#define FF_NOSENTMAIL    0x4000 /* Don't save in SentMail */

  /* Third party program flags in fldt.fs3rdParty */

#define FF3_SQ2MED_MASK  0x0003 /* Contact Dmitriy Grachjev (2:5020/328) */

  /* Message base type helpers */

#define MSGBASETYPE(fs)  ((fs) & FF_XMSGBASE)

#define IS_MSG(fs)       (MSGBASETYPE(fs) == FF_MSGSDM)
#define IS_SQU(fs)       (MSGBASETYPE(fs) == FF_MSGSQUISH)
#define IS_JAM(fs)       (MSGBASETYPE(fs) == FF_MSGJAM)
#define IS_HUD(fs)       (MSGBASETYPE(fs) == FF_MSGHUDSON)

  /* Folder type helpers */

#define FOLDERTYPE(fs)   ((fs) & FF_XFLDTYPE)

#define IS_NETMAIL(fs)   (FOLDERTYPE(fs) == FF_NETMAIL)
#define IS_ECHOMAIL(fs)  (FOLDERTYPE(fs) == FF_ECHOMAIL)
#define IS_GROUPMAIL(fs) (FOLDERTYPE(fs) == FF_GROUPMAIL)
#define IS_LOCALMAIL(fs) (FOLDERTYPE(fs) == FF_LOCALMAIL)

 /* Opus style message attributes */

 #define MSGPRIVATE 0x0001
 #define MSGCRASH   0x0002
 #define MSGREAD    0x0004
 #define MSGSENT    0x0008
 #define MSGFILE    0x0010
 #define MSGFWD     0x0020
 #define MSGORPHAN  0x0040
 #define MSGKILL    0x0080
 #define MSGLOCAL   0x0100
 #define MSGHOLD    0x0200
 #define MSGXX2     0x0400
 #define MSGFRQ     0x0800
 #define MSGRRQ     0x1000
 #define MSGCPT     0x2000
 #define MSGARQ     0x4000
 #define MSGURQ     0x8000

  /* MadMED message attributes in fldt.attr */

#define MA_PRVT  MSGPRIVATE     /* Private                 + */
#define MA_CRSH  MSGCRASH       /* Crash                   | These */
#define MA_READ  MSGREAD        /* Read                    | flags are */
#define MA_SENT  MSGSENT        /* Sent                    | standard for */
#define MA_FILE  MSGFILE        /* File attach             | FidoNet(tm) */
#define MA_TRAN  MSGFWD         /* Transit                 | message attr */
#define MA_ORPH  MSGORPHAN      /* Orphan                  | */
#define MA_KILL  MSGKILL        /* Kill/sent               | */
#define MA_LOCL  MSGLOCAL       /* Local                   | */
#define MA_HOLD  MSGHOLD        /* Hold                    | */
#define MA_RSV2  MSGXX2         /* Reserved                | */
#define MA_FREQ  MSGFRQ         /* File request            | */
#define MA_RREQ  MSGRRQ         /* Return receipt request  | */
#define MA_RRCP  MSGCPT         /* Confirmation receipt msg| */
#define MA_AREQ  MSGARQ         /* Audit request           | */
#define MA_UREQ  MSGURQ         /* File update request     + */
#define MA_SCAN  MSGSCANNED     /* Squish scanned          - *.SQ? only */
#define MA_VUID  MSGUID         /* Valid xmsg.umsgid       - *.SQ? only??? */
#define MA_FZON  0x80000000l    /* Zonegate route          + */
#define MA_FHUB  0x40000000l    /* Hub/host route          | These flags are */
#define MA_FIMM  0x20000000l    /* Immediate               | defined in the */
#define MA_FDIR  0x10000000l    /* Direct                  | ^aFLAGS kludge */
#define MA_FKFS  0x08000000l    /* Kill file sent          | proposed by */
#define MA_FTFS  0x04000000l    /* Truncate file sent      | FSC-0053 */
#define MA_FRRQ  0x02000000l    /* Mail unpack receipt req | */
#define MA_FCFM  0x01000000l    /* Mail read receipt req   | */
#define MA_FLOK  0x00800000l    /* Locked message          | */
#define MA_FAXS  0x00400000l    /* Archive/sent            | */
#define MA_FXMA  0x00200000l    /* Xmail                   + */
#define MA_FXXX  0xfff00000l    /* Mask for the ^aFLAGS attributes */

